@@ -129,12 +129,19 @@ int main(int argc, char *argv[])
     }
 
     //Test that mandatory locking works (this leaves file with EICAR data on disk)
-    if ( HardLockTest() == false )
+    int locktry = 0;
+    while ( HardLockTest() == false )
     {
         if (fd_tempfile > -1) while (close(fd_tempfile) < 0 && errno == EINTR);
         while (unlink(TempFileName) < 0 && (errno == EINTR || errno == EBUSY));
-        cout << "Exiting.." << endl;
-        exit(1);
+
+        if ( ++locktry > 3 )
+        {
+            cout << "Too many tries - Exiting.." << endl;
+            exit(1);
+        }
+
+        sleep(1);
     }
 
     //Tempfile descriptor not needed anymore
@@ -199,7 +206,19 @@ int main(int argc, char *argv[])
             LogFile::ErrorMessage("Signal HUP received, reloading scanners and lists\n");
 
             //Reload databases
-            if ( Scanners.ReloadDatabases() == true ) restartchilds = true;
+            int rl = Scanners.ReloadDatabases();
+
+            if ( rl == 1 )
+            {
+                restartchilds = true;
+            }
+            else if ( rl < 0 )
+            {
+                LogFile::ErrorMessage("FATAL: Library Scanners failed reloading databases, Exiting HAVP!");
+                killpg(getpgid(0), SIGTERM);
+                sleep(10);
+                exit(1);
+            }
 
             //Reload lists
             Whitelist.ReloadURLList( Params::GetConfigString("WHITELIST") );
@@ -213,10 +232,20 @@ int main(int argc, char *argv[])
         }
         else if ( time(NULL) > (LastRefresh + dbreload*60) ) //Time Refresh
         {
-            if ( Scanners.ReloadDatabases() == true )
+            //Reload databases
+            int rl = Scanners.ReloadDatabases();
+
+            if ( rl == 1 )
             {
                 restartchilds = true;
             }
+            else if ( rl < 0 )
+            {
+                LogFile::ErrorMessage("FATAL: Library Scanners failed database reload, stopping HAVP!");
+                killpg(getpgid(0), SIGTERM);
+                sleep(10);
+                exit(1);
+            }
 
             LastRefresh = time(NULL);
         }
@@ -258,6 +287,7 @@ int main(int argc, char *argv[])
                 //Create tempfile for scanning
                 if ( Scanners.InitTempFile() == false )
                 {
+                    Scanners.DeleteTempFile();
                     sleep(10);
                     exit(1);
                 }
