@@ -44,26 +44,26 @@ void Params::SetDefaults()
     SetConfig("SOURCE_ADDRESS",	"");
     SetConfig("PARENTPROXY",	"");
     SetConfig("PARENTPORT",	"0");
-    SetConfig("ACCESSLOG",	"/var/log/havp/access.log");
-    SetConfig("ERRORLOG",	"/var/log/havp/havp.log");
+    SetConfig("ACCESSLOG",	ACCESSLOG);
+    SetConfig("ERRORLOG",	ERRORLOG);
     SetConfig("LOG_OKS",	"true");
-    SetConfig("LOGLEVEL",	"1");
+    SetConfig("LOGLEVEL",	"0");
     SetConfig("USESYSLOG",	"false");
     SetConfig("SYSLOGNAME",	"havp");
     SetConfig("SYSLOGFACILITY",	"daemon");
     SetConfig("SYSLOGLEVEL",	"info");
     SetConfig("SCANIMAGES",	"true");
-    SetConfig("MAXSCANSIZE",	"0");
+    SetConfig("MAXSCANSIZE",	"5000000");
     SetConfig("KEEPBACKBUFFER",	"200000");
     SetConfig("KEEPBACKTIME",	"5");
     SetConfig("TRICKLING",	"30");
     SetConfig("WHITELISTFIRST",	"true");
-    SetConfig("WHITELIST",	"/usr/local/etc/havp/whitelist");
-    SetConfig("BLACKLIST",	"/usr/local/etc/havp/blacklist");
-    SetConfig("TEMPLATEPATH",	"/usr/local/etc/havp/templates/en");
+    SetConfig("WHITELIST",	WHITELISTFILE);
+    SetConfig("BLACKLIST",	BLACKLISTFILE);
+    SetConfig("TEMPLATEPATH",	TEMPLATEPATH);
     SetConfig("TEMPDIR",	"/var/tmp");
     SetConfig("SCANTEMPFILE",	"/var/tmp/havp/havp-XXXXXX");
-    SetConfig("PIDFILE",	"/var/run/havp/havp.pid");
+    SetConfig("PIDFILE",	PIDFILE);
     SetConfig("TRANSPARENT",	"false");
     SetConfig("RANGE",		"false");
     SetConfig("FORWARDED_IP",	"false");
@@ -74,8 +74,11 @@ void Params::SetDefaults()
     SetConfig("FAILSCANERROR",	"true");
     SetConfig("MAXDOWNLOADSIZE","0");
     SetConfig("SCANNERTIMEOUT",	"10");
+    SetConfig("IGNOREVIRUS",	"");
+//SCANNERS
     SetConfig("ENABLECLAMLIB","false");
         SetConfig("CLAMDBDIR","");
+        SetConfig("CLAMBLOCKBROKEN","false");
         SetConfig("CLAMBLOCKMAX","false");
         SetConfig("CLAMBLOCKENCRYPTED","false");
         SetConfig("CLAMMAXFILES","1000");
@@ -96,6 +99,7 @@ void Params::SetDefaults()
         SetConfig("FPROTSERVER","127.0.0.1");
     SetConfig("ENABLENOD32","false");
         SetConfig("NOD32SOCKET","/tmp/nod32d.sock");
+        SetConfig("NOD32VERSION","25");
     SetConfig("ENABLETROPHIE","false");
         SetConfig("TROPHIEMAXFILES","1000");
         SetConfig("TROPHIEMAXFILESIZE","10");
@@ -106,6 +110,8 @@ void Params::SetDefaults()
         SetConfig("AVASTSOCKET","/var/run/avast4/local.sock");
         SetConfig("AVASTSERVER","");
         SetConfig("AVASTPORT","5036");
+    SetConfig("ENABLEARCAVIR","false");
+        SetConfig("ARCAVIRSOCKET","/var/run/arcavird.socket");
 }
 
 bool Params::ReadConfig( string file )
