@@ -35,6 +35,7 @@
 #include "scanners/clamdscanner.h"
 #include "scanners/sophiescanner.h"
 #include "scanners/avastscanner.h"
+#include "scanners/arcavirscanner.h"
 
 #include <sys/types.h>
 #include <signal.h>
@@ -95,6 +96,10 @@ bool ScannerHandler::InitScanners()
     {
         VirusScanner.push_back(new AvastScanner);
     }
+    if ( Params::GetConfigBool("ENABLEARCAVIR") )
+    {
+        VirusScanner.push_back(new ArcavirScanner);
+    }
 
     if ( VirusScanner.size() == 0 )
     {
@@ -283,31 +288,45 @@ bool ScannerHandler::CreateScanners( SocketHandler &ProxyServerT )
     VirusMsg.clear();
     ErrorMsg.clear();
 
+    LastRequestURL = "";
+    CompleteTempFile = false;
+
     return true;
 }
 
 
 //Reload scanner databases
 //Called from main HAVP process only!
-bool ScannerHandler::ReloadDatabases()
+int ScannerHandler::ReloadDatabases()
 {
-    bool reloaded = false;
+    int reloaded = 0;
+    int failed = 0;
 
     for ( unsigned int i = 0; i < VirusScanner.size(); i++ )
     {
-        if ( VirusScanner[i]->ReloadDatabase() == true )
+        int rl = VirusScanner[i]->ReloadDatabase();
+
+        if ( rl == 1 )
         {
-            reloaded = true;
+            reloaded++;
+        }
+        else if ( rl < 0 )
+        {
+            failed++;
         }
     }
 
-    return reloaded;
+    if ( failed ) return -1;
+    if ( reloaded ) return 1;
+    return 0;
 }
 
 
 //Tell all scanners to start scanning again
 bool ScannerHandler::RestartScanners()
 {
+    if ( DeadScanner ) return false;
+
     int ret;
 
     for ( int i = 0; i < totalscanners; i++ )
@@ -316,7 +335,7 @@ bool ScannerHandler::RestartScanners()
 
         if (ret <= 0)
         {
-            LogFile::ErrorMessage("Detected dead %s process\n", Scanner[i].scanner_name.c_str());
+            LogFile::ErrorMessage("Detected crashed %s process (restart, pid: %d, lasturl: %s)\n", Scanner[i].scanner_name.c_str(), Scanner[i].scanner_pid, LastRequestURL.c_str());
             return false;
         }
     }
@@ -331,6 +350,9 @@ bool ScannerHandler::RestartScanners()
     VirusMsg.clear();
     ErrorMsg.clear();
 
+    LastRequestURL = "";
+    CompleteTempFile = false;
+
     return true;
 }
 
@@ -346,7 +368,7 @@ void ScannerHandler::ExitScanners()
 
         if (ret <= 0)
         {
-            LogFile::ErrorMessage("Detected dead %s process\n", Scanner[i].scanner_name.c_str());
+            LogFile::ErrorMessage("Detected crashed %s process (exit, pid: %d, lasturl: %s)\n", Scanner[i].scanner_name.c_str(), Scanner[i].scanner_pid, LastRequestURL.c_str());
         }
     }
 }
@@ -388,9 +410,10 @@ bool ScannerHandler::HasAnswer()
 
             if (ret <= 0) //Pipe was closed - or some bad error
             {
-                LogFile::ErrorMessage("Detected dead %s process\n", Scanner[i].scanner_name.c_str());
+                LogFile::ErrorMessage("Detected crashed %s process (hasanswer, pid: %d, lasturl: %s)\n", Scanner[i].scanner_name.c_str(), Scanner[i].scanner_pid, LastRequestURL.c_str());
+                DeadScanner = true;
 
-                ErrorMsg.push_back( Scanner[i].scanner_name_short + ": Scanner died" );
+                ErrorMsg.push_back( Scanner[i].scanner_name_short + ": Scanner crashed" );
 
                 //We still need to check other possible answers
                 continue;
@@ -407,7 +430,10 @@ bool ScannerHandler::HasAnswer()
                 Temp.erase(0,1);
                 if ( Temp == "" ) Temp = "Unknown";
 
-                VirusMsg.push_back( Scanner[i].scanner_name_short + ": " + Temp );
+                if ( IgnoredVirus( Temp ) == false )
+                {
+                    VirusMsg.push_back( Scanner[i].scanner_name_short + ": " + Temp );
+                }
 
                 //Check next descriptor
                 continue;
@@ -449,7 +475,9 @@ int ScannerHandler::GetAnswer()
 
         if (ret <= 0)
         {
-            LogFile::ErrorMessage("Detected dead %s process\n", Scanner[i].scanner_name.c_str());
+            LogFile::ErrorMessage("Detected crashed %s process (getanswer, pid: %d, lasturl: %s)\n", Scanner[i].scanner_name.c_str(), Scanner[i].scanner_pid, LastRequestURL.c_str());
+            DeadScanner = true;
+
             for ( int ii = 0; ii < totalscanners; ii++ )
             {
                 kill(Scanner[ii].scanner_pid, SIGKILL);
@@ -476,7 +504,7 @@ int ScannerHandler::GetAnswer()
         //If some scanner has timed out, kill them all
         if ( ret == 0 )
         {
-            LogFile::ErrorMessage("Error: Some scanner has timed out! Killing..\n");
+            LogFile::ErrorMessage("Error: Some scanner has timed out! (lasturl: %s)\n", LastRequestURL.c_str());
 
             for ( int i = 0; i < totalscanners; i++ )
             {
@@ -507,9 +535,10 @@ int ScannerHandler::GetAnswer()
 
                 if (ret <= 0) //Pipe was closed - or some bad error
                 {
-                    LogFile::ErrorMessage("Detected dead %s process\n", Scanner[i].scanner_name.c_str());
+                    LogFile::ErrorMessage("Detected crashed %s process (getanswer, pid: %d, lasturl: %s)\n", Scanner[i].scanner_name.c_str(), Scanner[i].scanner_pid, LastRequestURL.c_str());
+                    DeadScanner = true;
 
-                    ErrorMsg.push_back( Scanner[i].scanner_name_short + ": Scanner died" );
+                    ErrorMsg.push_back( Scanner[i].scanner_name_short + ": Scanner crashed" );
 
                     //We still need to check other possible answers
                     continue;
@@ -526,7 +555,10 @@ int ScannerHandler::GetAnswer()
                     Temp.erase(0,1);
                     if ( Temp == "" ) Temp = "Unknown";
 
-                    VirusMsg.push_back( Scanner[i].scanner_name_short + ": " + Temp );
+                    if ( IgnoredVirus( Temp ) == false )
+                    {
+                        VirusMsg.push_back( Scanner[i].scanner_name_short + ": " + Temp );
+                    }
 
                     //Check next descriptor
                     continue;
@@ -547,7 +579,10 @@ int ScannerHandler::GetAnswer()
     }
 
     //Virus found?
-    if ( VirusMsg.size() ) return 1;
+    if ( VirusMsg.size() )
+    {
+        return 1;
+    }
 
     //Return error only if all scanners had one and we want errors
     if ( (ErrorMsg.size() == (unsigned int)totalscanners) && Params::GetConfigBool("FAILSCANERROR") )
@@ -588,6 +623,47 @@ string ScannerHandler::GetAnswerMessage()
 }
 
 
+//Update last requested URL
+void ScannerHandler::LastURL( string URL )
+{
+    LastRequestURL = URL;
+}
+
+
+//Called if TempFile is fully complete, not cut off with MaxScanSize etc
+void ScannerHandler::HaveCompleteFile()
+{
+    CompleteTempFile = true;
+}
+
+
+//Check if Virus is whitelisted
+bool ScannerHandler::IgnoredVirus( string VirusName )
+{
+    //IGNOREVIRUS?
+    if ( IgnoredViruses.size() > 0 )
+    {
+        string Virus = UpperCase( VirusName );
+
+        for ( vector<string>::iterator IV = IgnoredViruses.begin(); IV != IgnoredViruses.end(); ++IV )
+        {
+            if ( Virus.find(*IV) != string::npos )
+            {
+                return true;
+            }
+        }
+    }
+
+    //CLAMBLOCKBROKEN? Ignore detection from incomplete file
+    if ( VirusName == "Broken.Executable" && CompleteTempFile == false )
+    {
+        return true;
+    }
+
+    return false;
+}
+
+
 //Initialize tempfile
 bool ScannerHandler::InitTempFile()
 {
@@ -708,6 +784,7 @@ bool ScannerHandler::ReinitTempFile()
     if (lseek(fd_tempfile, 0, SEEK_SET) < 0)
     {
         LogFile::ErrorMessage("Could not lseek Scannerfile: %s\n", strerror(errno));
+        DeleteTempFile();
         exit(1);
     }
 
@@ -716,6 +793,7 @@ bool ScannerHandler::ReinitTempFile()
         if (errno == EINTR) continue;
 
         LogFile::ErrorMessage("Could not truncate file: %s\n", strerror(errno));
+        DeleteTempFile();
         exit(1);
     }
 
@@ -725,6 +803,7 @@ bool ScannerHandler::ReinitTempFile()
         if (errno == EINTR) continue;
 
         LogFile::ErrorMessage("Could not write file: %s\n", strerror(errno));
+        DeleteTempFile();
         exit(1);
     }
 #endif
@@ -738,6 +817,7 @@ bool ScannerHandler::ReinitTempFile()
         if (errno == EINTR) continue;
 
         LogFile::ErrorMessage("Could not fchmod() Scannerfile: %s\n", strerror(errno));
+        DeleteTempFile();
         exit(1);
     }
 
@@ -763,6 +843,7 @@ bool ScannerHandler::ReinitTempFile()
         }
 
         LogFile::ErrorMessage("Could not lock Scannerfile: %s\n", strerror(errno));
+        DeleteTempFile();
         exit(1);
     }
 #endif
@@ -770,6 +851,7 @@ bool ScannerHandler::ReinitTempFile()
     if (lseek(fd_tempfile, 0, SEEK_SET) < 0)
     {
         LogFile::ErrorMessage("Could not lseek Scannerfile: %s\n", strerror(errno));
+        DeleteTempFile();
         exit(1);
     }
 
@@ -818,6 +900,7 @@ bool ScannerHandler::TruncateTempFile( long long ContentLengthT )
         if (errno == EINTR) continue;
 
         LogFile::ErrorMessage("Could not truncate Scannerfile: %s\n", strerror(errno));
+        DeleteTempFile();
         exit(1);
     }
 
@@ -872,10 +955,8 @@ bool ScannerHandler::ExpandTempFile( string &dataT, bool unlockT )
     return true;
 }
 
-#ifndef NOMAND
 bool ScannerHandler::ExpandTempFileRange( string &dataT, long long offset )
 {
-    struct flock lock;
     int total_written = 0;
     int len = dataT.length();
     int ret;
@@ -902,6 +983,9 @@ bool ScannerHandler::ExpandTempFileRange( string &dataT, long long offset )
     //if (lseek(fd_tempfile, origpos, SEEK_SET) < 0) return false;
     if (lseek(fd_tempfile, 0, SEEK_SET) < 0) return false;
 
+#ifndef NOMAND
+    struct flock lock;
+
     lock.l_type   = F_UNLCK;
     lock.l_start  = offset;             // byte-offset
     lock.l_whence = SEEK_SET;
@@ -913,10 +997,10 @@ bool ScannerHandler::ExpandTempFileRange( string &dataT, long long offset )
         LogFile::ErrorMessage("Could not unlock file: %s\n", TempFileName);
         return false;
     }
+#endif
 
     return true;
 }
-#endif
 
 //Constructor
 ScannerHandler::ScannerHandler()
@@ -931,6 +1015,29 @@ ScannerHandler::ScannerHandler()
         //10 minutes as failsafe
         ScannerTimeout = 600;
     }
+
+    //Whitelisted viruses
+    if ( Params::GetConfigString("IGNOREVIRUS") != "" )
+    {
+        string Tokens = UpperCase( Params::GetConfigString("IGNOREVIRUS") );
+        string::size_type Position;
+
+        while ( (Position = Tokens.find(" ")) != string::npos )
+        {
+            if ( Position == 0 )
+            {
+                Tokens.erase( 0, 1 );
+                continue;
+            }
+
+            IgnoredViruses.push_back( Tokens.substr( 0, Position ));
+            Tokens.erase( 0, Position + 1 );
+        }
+
+        IgnoredViruses.push_back( Tokens );
+    }
+
+    DeadScanner = false;
 }
 
 //Destructor
