@@ -29,7 +29,11 @@ bool ClamLibScanner::InitDatabase()
 
     LogFile::ErrorMessage("ClamAV: Using database directory: %s\n", dbdir);
 
+#ifdef CL_DB_STDOPT
+    if ( (ret = cl_load(dbdir, &root, &no, CL_DB_STDOPT)) != 0 )
+#else
     if ( (ret = cl_loaddbdir(dbdir, &root, &no)) != 0 )
+#endif
     {
         LogFile::ErrorMessage("ClamAV: Could not load database: %s\n", cl_strerror(ret));
         return false;
@@ -52,7 +56,7 @@ bool ClamLibScanner::InitDatabase()
 }
 
 
-bool ClamLibScanner::ReloadDatabase()
+int ClamLibScanner::ReloadDatabase()
 {
     int ret = cl_statchkdir(&dbstat);
 
@@ -65,17 +69,21 @@ bool ClamLibScanner::ReloadDatabase()
 
         cl_settempdir(Params::GetConfigString("TEMPDIR").c_str(), 0);
 
+#ifdef CL_DB_STDOPT
+        if ( (ret = cl_load(dbdir, &root, &no, CL_DB_STDOPT)) != 0 )
+#else
         if ( (ret = cl_loaddbdir(dbdir, &root, &no)) != 0 )
+#endif
         {
             LogFile::ErrorMessage("ClamAV: Could not reload database: %s\n", cl_strerror(ret));
-            return false;
+            return -1;
         }
 
         if ( (ret = cl_build(root)) != 0 )
         {
             LogFile::ErrorMessage("ClamAV: Database initialization error: %s\n", cl_strerror(ret));
             cl_free(root);
-            return false;
+            return -1;
         }
 
         LogFile::ErrorMessage("ClamAV: Reloaded %d signatures (engine %s)\n", no, cl_retver());
@@ -85,14 +93,14 @@ bool ClamLibScanner::ReloadDatabase()
         memset(&dbstat, 0, sizeof(struct cl_stat));
         cl_statinidir(dbdir, &dbstat);
 
-        return true;
+        return 1;
     }
     else if ( ret != 0 )
     {
         LogFile::ErrorMessage("ClamAV: Error on database check: %s\n", cl_strerror(ret));
     }
 
-    return false;
+    return 0;
 }
 
 
@@ -138,7 +146,17 @@ void ClamLibScanner::FreeDatabase()
 //Constructor
 ClamLibScanner::ClamLibScanner()
 {
-    ScannerName = "ClamAV Library Scanner";
+    ScannerName = cl_retver();
+
+    if ( MatchSubstr( ScannerName, "devel", -1 ) )
+    {
+        ScannerName = "ClamAV Library Scanner (devel)";
+    }
+    else
+    {
+        ScannerName = "ClamAV Library Scanner";
+    }
+
     ScannerNameShort = "ClamAV";
 
     memset(&dbdir, 0, sizeof(dbdir));
@@ -163,6 +181,10 @@ ClamLibScanner::ClamLibScanner()
     {
         scanopts = scanopts | CL_SCAN_BLOCKENCRYPTED;
     }
+    if ( Params::GetConfigBool("CLAMBLOCKBROKEN") )
+    {
+        scanopts = scanopts | CL_SCAN_BLOCKBROKEN;
+    }
 
     //Set up archive limits
     memset(&limits, 0, sizeof(limits));
