@@ -25,6 +25,7 @@ class NOD32Scanner : public GenericScanner {
 private:
 
 string Agent;
+int Version;
 
 string ScannerCmd;
 
@@ -37,9 +38,11 @@ char Ready[2];
 public:
 
 bool InitDatabase();
-bool ReloadDatabase();
+int ReloadDatabase();
 void FreeDatabase();
 string Scan( const char *FileName );
+string ScanV25( const char *FileName );
+string ScanV21( const char *FileName );
 
 NOD32Scanner();
 ~NOD32Scanner();
