@@ -111,7 +111,7 @@ unsigned int cur_patt;
 public:
 
 bool InitDatabase();
-bool ReloadDatabase();
+int ReloadDatabase();
 void FreeDatabase();
 string Scan( const char *FileName );
 
